﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using WinMapSamples.Properties;

namespace WinMapSamples
{
    public partial class Main : Form
    {
        public Main()
        {
            InitializeComponent();
        }

        private void Main_Load(object sender, EventArgs e)
        {
            // set map unit
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;

            // add overlay
            winformsMap1.Overlays.Add(new WorldMapKitWmsDesktopOverlay());
           
            // set map extent
            winformsMap1.CurrentExtent = new RectangleShape(-122.437392, 47.626353, -71.089115, 25.787676);

            winformsMap1.Refresh();
        }

        private void button_Click(object sender, EventArgs e)
        {
            Button button = sender as Button;
            if (button != null)
            {
                switch (button.Name)
                {
                    case "btnTrackNormal":
                        winformsMap1.TrackOverlay.TrackMode = TrackMode.None;
                        break;
                    case "btnTrackPoint":
                        winformsMap1.TrackOverlay.TrackMode = TrackMode.Point;
                        break;
                    case "btnTrackLine":
                        winformsMap1.TrackOverlay.TrackMode = TrackMode.Line;
                        break;
                    case "btnTrackRectangle":
                        winformsMap1.TrackOverlay.TrackMode = TrackMode.Rectangle;
                        break;
                    case "btnTrackSquare":
                        winformsMap1.TrackOverlay.TrackMode = TrackMode.Square;
                        break;
                    case "btnTrackPolygon":
                        winformsMap1.TrackOverlay.TrackMode = TrackMode.Polygon;
                        break;
                    case "btnTrackCircle":
                        winformsMap1.TrackOverlay.TrackMode = TrackMode.Circle;
                        break;
                    case "btnTrackEllipse":
                        winformsMap1.TrackOverlay.TrackMode = TrackMode.Ellipse;
                        break;
                    case "btnTrackEdit":
                        winformsMap1.TrackOverlay.TrackMode = TrackMode.None;
                        foreach (Feature feature in winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures)
                        {
                            winformsMap1.EditOverlay.EditShapesLayer.InternalFeatures.Add(feature);
                        }
                        winformsMap1.EditOverlay.CalculateAllControlPoints();
                        winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Clear();

                        winformsMap1.Refresh(new Overlay[] { winformsMap1.EditOverlay, winformsMap1.TrackOverlay });
                        break;
                    case "btnTrackDelete":
                        winformsMap1.TrackOverlay.TrackMode = TrackMode.None;
                        winformsMap1.EditOverlay.EditShapesLayer.InternalFeatures.Clear();
                        winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Clear();
                        winformsMap1.Refresh(new Overlay[] { winformsMap1.EditOverlay, winformsMap1.TrackOverlay });
                        break;
                    default:
                        winformsMap1.TrackOverlay.TrackMode = TrackMode.None;
                        break;
                }
            }
        }

        private void btn_CreateNewPointFile(object sender, EventArgs e)
        {
            // create columns for the new shapefile
            Collection<DbfColumn> newColumns = new Collection<DbfColumn>();
            newColumns.Add(new DbfColumn("COLUMN1", DbfColumnType.Character, 32, 0));

            string newShapefilePathName = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "newPoints.shp");
            ShapeFileFeatureSource.CreateShapeFile(ShapeFileType.Point, newShapefilePathName, newColumns);

            // add features to the new shapefile
            ShapeFileFeatureLayer newFeatureLayer = new ShapeFileFeatureLayer(newShapefilePathName, ShapeFileReadWriteMode.ReadWrite);
            newFeatureLayer.Open();

            // begin transaction
            newFeatureLayer.EditTools.BeginTransaction();

            int totalCount = 0;
            var pointFeatures = winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Where(f => f.GetShape() is PointShape).ToList();
            for (int i = 0; i < pointFeatures.Count; i++)
            {
                totalCount++;
                Feature point = pointFeatures[i];
                point.ColumnValues.Add("COLUMN1", i.ToString());
                newFeatureLayer.EditTools.Add(point);
                if (totalCount % 1000 == 0)
                {
                    newFeatureLayer.EditTools.CommitTransaction();
                    newFeatureLayer.EditTools.BeginTransaction();
                }
            }

            // commit transaction
            newFeatureLayer.EditTools.CommitTransaction();
            newFeatureLayer.Close();

            MessageBox.Show(string.Format("Create shapefile: {0} success.", newShapefilePathName));
        }

        private void btn_CreateNewLineFile(object sender, EventArgs e)
        {
            // create columns for the new shapefile
            Collection<DbfColumn> newColumns = new Collection<DbfColumn>();
            newColumns.Add(new DbfColumn("COLUMN1", DbfColumnType.Character, 32, 0));

            string newShapefilePathName = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "newLines.shp");
            ShapeFileFeatureSource.CreateShapeFile(ShapeFileType.Polyline, newShapefilePathName, newColumns);

            // add features to the new shapefile
            ShapeFileFeatureLayer newFeatureLayer = new ShapeFileFeatureLayer(newShapefilePathName, ShapeFileReadWriteMode.ReadWrite);
            newFeatureLayer.Open();

            // begin transaction
            newFeatureLayer.EditTools.BeginTransaction();

            int totalCount = 0;
            var lineFeatures = winformsMap1.TrackOverlay.TrackShapeLayer.InternalFeatures.Where(f => f.GetShape() is LineShape).ToList();
            for (int i = 0; i < lineFeatures.Count; i++)
            {
                totalCount++;
                Feature line = lineFeatures[i];
                line.ColumnValues.Add("COLUMN1", i.ToString());
                newFeatureLayer.EditTools.Add(line);
                if (totalCount % 1000 == 0)
                {
                    newFeatureLayer.EditTools.CommitTransaction();
                    newFeatureLayer.EditTools.BeginTransaction();
                }
            }

            // commit transaction
            newFeatureLayer.EditTools.CommitTransaction();
            newFeatureLayer.Close();

            MessageBox.Show(string.Format("Create shapefile: {0} success.", newShapefilePathName));
        }

        private void btn_CreateNew(object sender, EventArgs e)
        {
            ShapeFileFeatureLayer citiesFeatureLayer = new ShapeFileFeatureLayer(@"..\..\App_Data\MajorCities.shp");

            // open layer before query or edit.
            citiesFeatureLayer.Open();

            // query features it's as same as citiesFeatureLayer.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns);
            Collection<Feature> cities = citiesFeatureLayer.QueryTools.GetAllFeatures(ReturningColumnsType.AllColumns);
            Collection<FeatureSourceColumn> oldColumns = citiesFeatureLayer.QueryTools.GetColumns();
            Collection<DbfColumn> newColumns = new Collection<DbfColumn>();

            // add the exist columns
            foreach (FeatureSourceColumn column in oldColumns)
            {
                newColumns.Add((DbfColumn)column);
            }
            // add the new column to newsColumns
            newColumns.Add(new DbfColumn("NewColumnName", DbfColumnType.Character, 32, 0));

            string newShapefilePathName = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "newMajorCities.shp");
            // create new shapefile
            ShapeFileFeatureSource.CreateShapeFile(citiesFeatureLayer.GetShapeFileType(), newShapefilePathName, newColumns);
            citiesFeatureLayer.Close();

            // add features to the new shapefile
            ShapeFileFeatureLayer newFeatureLayer = new ShapeFileFeatureLayer(newShapefilePathName, ShapeFileReadWriteMode.ReadWrite);
            newFeatureLayer.Open();

            // begin transaction
            newFeatureLayer.EditTools.BeginTransaction();

            int totalCount = 0;
            for (int i = 0; i < cities.Count; i++)
            {
                totalCount++;
                Feature city = cities[i];
                city.ColumnValues.Add("NewColumnName", i.ToString());
                newFeatureLayer.EditTools.Add(city);
                if (totalCount % 1000 == 0)
                {
                    newFeatureLayer.EditTools.CommitTransaction();
                    newFeatureLayer.EditTools.BeginTransaction();
                }
            }


            // commit transaction
            newFeatureLayer.EditTools.CommitTransaction();
            newFeatureLayer.Close();

            MessageBox.Show(string.Format("Create shapefile: {0} success.", newShapefilePathName));
        }

        private void btn_Animation(object sender, EventArgs e)
        {
            ShapeFileFeatureSource citiesFeatureSource = new ShapeFileFeatureSource(@"..\..\App_Data\MajorCities.shp");
            FeatureSourceMarkerOverlay markerOverlay = new FeatureSourceMarkerOverlay(citiesFeatureSource);
            markerOverlay.MapControl = winformsMap1;
            markerOverlay.ZoomLevelSet.ZoomLevel01.CustomMarkerStyle = new AnimationMarkerStyle()
            {
                AnimationImage = Resources.animationpoint,
                Width = 20,
                Height = 34,
                YOffset = -17
            };
            markerOverlay.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            winformsMap1.Overlays.Add(markerOverlay);

            winformsMap1.Refresh(markerOverlay);
        }

    }
}
