﻿using System;
using System.Drawing;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;

namespace ThinkGeo.MapSuite.DesktopEdition
{
    public class AnimationMarker : Marker
    {
        private Bitmap animation;
        private bool currentlyAnimating;
        private Image image;

        public AnimationMarker()
            : this(new PointShape())
        { }

        public AnimationMarker(double worldX, double worldY)
            : this(new PointShape(worldX, worldY))
        { }

        public AnimationMarker(PointShape position)
            : base(position)
        {
 
        }

        public Bitmap Animation
        {
            get { return animation; }
            set { animation = value; }
        }

        private void OnFrameChanged(object o, EventArgs e)
        {
            //Force a call to the Paint event handler. 
            this.Invalidate();
        }

        private Image GetImage() 
        {
            if (image == null)
            {
                image = new Bitmap(this.Width, this.Height);
                Image = image;
            }
            return Image;
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            BeginAnimation();
            ImageAnimator.UpdateFrames();

            Graphics g = Graphics.FromImage(GetImage());
            g.DrawImage(animation, new Point());
        }

        private void BeginAnimation()
        {
            if (!currentlyAnimating)
            {
                ImageAnimator.Animate(animation, new EventHandler(OnFrameChanged));
                currentlyAnimating = true;
            }
        }

        private void StopAnimation()
        {
            if (currentlyAnimating)
            {
                ImageAnimator.StopAnimate(animation, OnFrameChanged);
                currentlyAnimating = false;
            }
        }

    }
}
